#import "checksum.h"
#import <objc/List.h>

@implementation NSMutableArray(Compatibility)

- (id)initFromList:(id)aList
{
    int i, count;

    if ([aList isKindOf:[List class]]) {
        count = [aList count];
        // done in Bogus init [self initWithCapacity:count];
        for (i = 0; i < count; i++) {
            [self addObject:[aList objectAt:i]];
        }
    } else if ([aList isKindOf:[NSArray class]]) {
        return [self initWithArray:aList];
    } else {
        /* should probably raise */
    }

    return self;
}
@end

@implementation BogusList

+ (void)initialize
{
    (void)[BogusList setVersion:(int)OS_VERSION];   // 100 is OS version
}

- init
{
    [super init];
    array = [[NSMutableArray allocWithZone:[self zone]]initWithCapacity:1];
    return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
    [aCoder encodeValuesOfObjCTypes:"@",&array];
}


- (id)initWithCoder:(NSCoder *)aDecoder
{
    [aDecoder decodeValuesOfObjCTypes:"@", &array];
    return self;
}

- (id)initWithCoderExpectingList:(NSCoder *)aDecoder
{
// this doesn't work - almost but not quite
// we really need to know how List does it's write:
// for various versions...

    int count;
    [aDecoder decodeValuesOfObjCTypes:"i", &count];
    if (count > 0) {
        id *list;
        int i;
        list = NSZoneMalloc([self zone], (count) * sizeof(id));
        [aDecoder decodeArrayOfObjCType:"@" count:count at:list];
        for (i = 0; i < count; i++) {
            [array addObject:list[i]]; // addObject causes a retain
        }
        NSZoneFree([self zone],list);
    }
    return self;
}


- (unsigned)count { return [array count]; }
- (id)objectAtIndex:(unsigned)index { return [array objectAtIndex:index]; }
- (unsigned)indexOfObject:(id)obj { return [array indexOfObject:obj]; }
- (id)lastObject { return [array lastObject]; }
- (void)makeObjectsPerform:(SEL)aSel { [array makeObjectsPerform:aSel]; }
- (void)makeObjectsPerform:(SEL)aSel withObject:(id)argument
{	[array makeObjectsPerform:aSel withObject:argument]; }
- (void)removeAllObjects { [array removeAllObjects]; }
- (void)removeObject:(id)anObject { [array removeObject:anObject]; }
- (void)addObject:(id)anObject { [array addObject:anObject]; }
- (void)insertObject:(id)anObject atIndex:(unsigned)index
{  [array insertObject:anObject atIndex:index]; }
- (void)removeLastObject { [array removeLastObject]; }
- (void)removeObjectAtIndex:(unsigned)i { [array removeObjectAtIndex:i]; }
- (void)replaceObjectAtIndex:(unsigned)index withObject:(id)anObject
{ [array replaceObjectAtIndex:index withObject:anObject]; }

@end